#include "PokktNativeManager.h"
#include "PokktDelegate.h"

// pokkt event definitions
const std::string PokktNativeManager::VideoClosedEvent = "videoClosed";
const std::string PokktNativeManager::VideoDisplayedEvent = "videoDisplayed";
const std::string PokktNativeManager::VideoSkippedEvent = "videoSkipped";
const std::string PokktNativeManager::VideoCompletedEvent = "videoCompleted";
const std::string PokktNativeManager::VideoGratifiedEvent = "videoGratified";
const std::string PokktNativeManager::DownloadCompletedEvent = "downloadCompleted";
const std::string PokktNativeManager::DownloadFailedEvent = "downloadFailed";
const std::string PokktNativeManager::CoinResponseEvent = "coinResponse";
const std::string PokktNativeManager::CoinResponseWithTransIdEvent = "coinResponseWithTrId";
const std::string PokktNativeManager::CoinResponseFailedEvent = "coinResponseFailed";
const std::string PokktNativeManager::CampaignAvailabilityEvent = "campaignAvailability";
const std::string PokktNativeManager::OfferwallClosedEvent = "offerwallClosedEvent";

bool PokktNativeManager::_isDebugEnabled = true;
_STL::map<std::string , PokktNativeManager::ListenerTargetPair*> PokktNativeManager::_eventMap;


void PokktNativeManager::handleNativeMessage(const char* operation, const char* param)
{
    POKKTLOG("[POKKT-CPP] message received! operation: %s, param: %s", operation, param);
    
    ListenerTargetPair *l = (_eventMap)[std::string(operation)];
    if (l != NULL)
    {
        EVT_SELECTOR listener = l->listener;
        PokktDelegate *target = l->target;
        (target->*listener)(std::string(param));
    }
    else
    {
        POKKTLOG("l is null");
    }
}

void PokktNativeManager::setListener(std::string eventName, EVT_SELECTOR listener, PokktDelegate *target)
{
    ListenerTargetPair *pair = new ListenerTargetPair();
    pair->listener = listener;
    pair->target = target;
    _eventMap[eventName] = pair;
}

void PokktNativeManager::removeListener(std::string eventName)
{
    if (_eventMap.size() == 0)
        return;
    
    _STL::map<std::string, ListenerTargetPair*>::iterator it = _eventMap.find(eventName);
    if (it != _eventMap.end())
    {
        delete it->second;
        _eventMap.erase(it);
    }
}

void PokktNativeManager::removeAllListeners()
{
    if (_eventMap.size() == 0)
        return;
    
    while (_eventMap.size() > 0)
    {
        _STL::map<std::string, ListenerTargetPair*>::iterator it = _eventMap.begin();
        removeListener(it->first);
    }
}

static std::string getParamString(PokktMarmaladeConfig *pokktMarmaladeConfig)
{
    std::string integrationType = "";
    switch (pokktMarmaladeConfig->getIntegrationType()) {
        case PokktMarmaladeConfig::INTEGRATION_TYPE_ALL:
            integrationType = "0";
            break;
        case PokktMarmaladeConfig::INTEGRATION_TYPE_OFFERWALL:
            integrationType = "1";
            break;
        case PokktMarmaladeConfig::INTEGRATION_TYPE_VIDEO:
            integrationType = "2";
            break;
        default:
            integrationType = "0";
            break;
    }
    std::string paramString =
    pokktMarmaladeConfig->getSecurityKey() + "," + pokktMarmaladeConfig->getApplicationId() + "," + integrationType + "," + (pokktMarmaladeConfig->getAutoCacheVideo() ? "true" : "false");
    
    return paramString;
}

void PokktNativeManager::setParams(PokktMarmaladeConfig *pokktMarmaladeConfig)
{
    PokktExtension::performOperation(PokktExtension::SetParams, getParamString(pokktMarmaladeConfig));
    
    PokktExtension::performOperation(PokktExtension::SetDebug, _isDebugEnabled ? "true" : "false");
}

// common methods

void PokktNativeManager::initPokkt(PokktMarmaladeConfig *pokktMarmaladeConfig)
{
    PokktNativeExtensionRegister(OPERATION_EVENT, (s3eCallback)PokktExtension::PokktCallbackReceived, NULL);
    setParams(pokktMarmaladeConfig);
}

void PokktNativeManager::setDebug(bool value)
{
    _isDebugEnabled = value;
    PokktExtension::performOperation(PokktExtension::SetDebug, _isDebugEnabled ? "true" : "false");
}

bool PokktNativeManager::getDebug()
{
    return _isDebugEnabled;
}

void PokktNativeManager::showLog(std::string message)
{
    PokktExtension::performOperation(PokktExtension::ShowLog, message);
}

void PokktNativeManager::showToast(std::string message)
{
    PokktExtension::performOperation(PokktExtension::ShowToast, message);
}

// session methods

void PokktNativeManager::startSession(PokktMarmaladeConfig *pokktMarmaladeConfig)
{
    setParams(pokktMarmaladeConfig);
    PokktExtension::performOperation(PokktExtension::StartSession);
}

void PokktNativeManager::endSession()
{
    PokktExtension::performOperation(PokktExtension::EndSession);
}

// OfferWall methods

void PokktNativeManager::getCoins(PokktMarmaladeConfig *pokktMarmaladeConfig)
{
    PokktExtension::performOperation(PokktExtension::GetCoins, pokktMarmaladeConfig->getOfferWallAssetValue());
}

void PokktNativeManager::getPendingCoins()
{
    PokktExtension::performOperation(PokktExtension::GetPendingCoins);
}

void PokktNativeManager::checkOfferWallCampaign()
{
    PokktExtension::performOperation(PokktExtension::CheckOfferWallCampaign);
}

// Video Methods

void PokktNativeManager::getVideo(PokktMarmaladeConfig *pokktMarmaladeConfig)
{
    pokktMarmaladeConfig->setIncentivised(true);
    PokktExtension::performOperation(PokktExtension::GetVideo, pokktMarmaladeConfig->getScreenName());
}

void PokktNativeManager::getVideoNonIncent(PokktMarmaladeConfig *pokktMarmaladeConfig)
{
    pokktMarmaladeConfig->setIncentivised(false);
    PokktExtension::performOperation(PokktExtension::GetVideoNonIncent, pokktMarmaladeConfig->getScreenName());
}

void PokktNativeManager::cacheVideoCampaign()
{
    PokktExtension::performOperation(PokktExtension::CacheVideoCampaign);
}

bool PokktNativeManager::isVideoAvailable()
{
    return PokktExtension::isVideoAvailable();
}

double PokktNativeManager::getVideoVc()
{
    return PokktExtension::getVideoVc();
}

std::string PokktNativeManager::getPokktSDKVersion(){
    return PokktExtension::getPokktSDKVersion();
}

//new method

void PokktNativeManager::sendAppInfo()
{
    PokktExtension::performOperation(PokktExtension::SendAppInfo);
}

void PokktNativeManager::exportLogs()
{
    PokktExtension::performOperation(PokktExtension::ExportLog);
}


void PokktNativeManager::terminatePokktNativeManager()
{
    PokktNativeExtensionUnRegister(OPERATION_EVENT, (s3eCallback)PokktExtension::PokktCallbackReceived);
}
